import {get, set, set_storage} from "./storage"

class Store {
  constructor(options){

    this.topic = 'dudy-' + options.topic
    this.initialState = options.initialState 
    this.persist = options.persist || false
    this.handlers = []
    this.store_handlers = []
    this.__init_promise = this.init().then( () => {
      return new Promise( (resolve, reject) => {
        if (!this.state) { // 从persist中没有获得初始值，通过reducer获取
          let nextState = this.state || {}
          this.handlers.forEach(h => {
            const next = h(this.state, {})
            if (next) {
              nextState = { ...nextState, ...next }
            }
          })
          this.state = nextState
        }
        resolve()

      })
    })

  }

  
  /**
   * 监听store内容变化
   * @param  handler 
   */
  subscribe( handler ) {
    this.store_handlers.push(handler)
    return () => {
      this.store_handlers = this.store_handlers.filter(x => x !== handler)
    }
  }

  /**
   * 定义store中的内容，如何到达下一个状态
   * 类似redux的reducer
   * @param handler 
   */
  next( handler ) {
    this.handlers.push(handler)

    // return unsubscribe
    return () => {
      this.handlers = this.handlers.filter(x => x !== handler)
    }
  }

  /**
   * 初始化store
   */
  async init(){
    this.state = this.initialState
    if(this.persist) {
      const str = await get(this.topic)
      if (str) {
        try{
          this.state = JSON.parse(str) 
        }catch(ex){
          this.state = this.initialState
        }
      }

    }

   
  }


  /**
   * 获取状态
   */
  get_state(){
    return this.state
  }

  /**
   * 设置状态，内部使用
   */
  __set_state = async (nextState) => {
    if(this.state !== nextState) {
      if(this.persist) {
        await set(this.topic, JSON.stringify(nextState))
      }

      this.state = nextState
      this.store_handlers.forEach(h => {
        h(this.state)
      })
    }


  }


  /**
   * 配置一个store
   */
  static config(options) {
    const __store = new Store(options)
    global.store.push(options.topic, __store)
  }

  get(name) {
    if(this.state) {
      return this.state[name]
    }
  }
}

module.exports = Store 