import Store from "./Store"
/**
 * 方便用户使用，将所有的store集中管理
 */
class StoreManager {
  constructor(){
    this.stores = {}
  }

  /**
   * 将一个事件源变成触发器
   * 每次事件发生时调用dispatch
   */
  source = (trigger, eventMapper) => {
    trigger( function() {
      const action = eventMapper.apply(null, arguments)
      store.dispatch(action)
    })    
  }

  dispatch = async (action) => {
    if(typeof action === 'string') {
      action = {type : action}
    }

    for(let key in this.stores) {
      const store = this.stores[key]
      let nextState = store.get_state()
      store.handlers.forEach(h => {
        const next = h(nextState, action)
        if (next) {
          nextState = { ...nextState, ...next }
        }
      })

      await store.__set_state(nextState)
    }
  }

  push(topic, proxy) {
    this.stores[topic] = proxy
  }

  /**
   * 获取store
   * 如果没有就创建
   */
  get(topic) {
    if (typeof topic !== 'string') { return }
    if (topic in this.stores) {
      return this.stores[topic]
    }

    Store.config({
      topic
    })
    return this.stores[topic]
  }
}


/**
 * 保证StoreManager是一个单例
 */
function init() {
  const storeMgr = new StoreManager()

  global.store = storeMgr 
  if(typeof window !== 'undefined') {
    window.store = global.store
  }
  return StoreManager 
}


module.exports = init()